#!/bin/sh
# 
# A simple prep script for setting up an encrypted linux root partition from 
# a Fedora 6+ installation CD.
#
# $Id: prep.sh,v 1.3 2007/06/06 04:48:58 root Exp root $
#

VERSION='$Id: prep.sh,v 1.3 2007/06/06 04:48:58 root Exp root $'

if [ ! -d /tmp/removable ] || ! mount | grep -q /tmp/removable ; then
	echo "Removable media not detected, proceed anyway? (yes/no) "
	while read answer ; do 
		case $answer in 
		y*)
			break 1
			;;
		n*)
			exit 1
			;;
		*)
			echo -n "yes/no? "
		esac
	done
fi
	
case `uname -r` in
*fc7)
	RPM_PATH=/mnt/source/Fedora/
	FEDORA=7
	;;
*fc6)
	RPM_PATH=/mnt/source/Fedora/RPMS
	FEDORA=6
	;;
*)
	unset FEDORA
	unset RPM_PATH
	echo "Oh my!  What are you running?"
	exit 9
esac

echo "Creating /tmp/encrypt to extract modules"
mkdir -p /tmp/encrypt/patches

cp -a fc${FEDORA}/* /tmp/encrypt/patches/
cd /tmp/encrypt 

confirm=no
until [ "$confirm" == "yes" ] ; do 
	echo -n "Detected ${HOSTTYPE}. What architecture are you? "
	while read answer ; do 
		case $answer in
			i?86)
				arch=i586
				break 1
				;;
			x86_64)
				arch=$answer
				break 1
				;;
			*)
				arch=$answer
				if [ -n "${arch}" ] ; then
					break 1
				fi
				;;
		esac
		echo -n "Detected ${HOSTTYPE}. What architecture are you? "
	done
	echo .
	
	echo -n "Is ${arch} close enough (yes/no) ? "
	while read confirm ; do 
		case $confirm in 
		y*)
			break 2
			;;
		n*)
			break
			;;
		esac
		echo .
		echo -n "Is ${arch} close enough (yes/no) ? "
	done
done


echo "Extracting RPMS from base installation"
for i in `find ${RPM_PATH} -name 'module-init*' -o -name 'kernel-*'${arch}'.rpm' -o -name 'cryptsetup-*' | grep -v '\-devel\-'` ; do
	rpm2cpio $i | cpio -idum   
done

for i in bin sbin usr/bin usr/sbin ; do 
	PATH=/tmp/encrypt/$i:$PATH
done

export PATH

for i in lib/modules/*/kernel/crypto/* ; do
	insmod $i
done

insmod lib/modules/*/kernel/drivers/md/dm-crypt.ko

echo "Begin creating your partitions and/or LV's.  When done, type exit"

echo .
PS1="partitions# " sh
echo .

echo "Begin your installation in Anaconda.  Do not select REBOOT from the 
installer.   Come back to this shell when Anaconda asks you if you want
to reboot.   Then, type Go!"

while read answer ; do 
	case $answer in 
	Go!)
		break 2
		;;
	*)
		echo -n "Go! or ^C : "
		;;
	esac
done

echo .
echo 'Edit /mnt/sysimage/etc/sysconfig/mkinitrd and include 
the mkinitrd commands to mount your particular installation. 
When done, type exit.

Hint: vi /mnt/sysimage/etc/sysconfig/mkinitrd
CRYPTODEVICES="<device>"
-or-
CRYPTODEVICES="<device>:/path/to/embedded/key"
-or-
CRYPTODEVICES="<device>@<keydevice>:/path/to/key"

CRYPTOMODS="<ALL>|<DEFAULT>|<specific>"
'
cat >> /mnt/sysimage/etc/sysconfig/mkinitrd <<!
CRYPTOMODS="DEFAULT"
CRYPTODEVICES=""
BINARIES=""
!

echo .
PS1="mkinitrd# " sh
echo .

echo "Edit /mnt/sysimage/etc/crypttab, if needed.  DO NOT add a line for your 
root device.

When you are done, type exit!"

echo .
PS1="crypttab# " sh
echo .

echo "You are ready for the final stage.  If you want to check one more time
type wait. If you are done, type Go!"

while read answer ; do 
	echo -n "[check! or Go!] "
	case $answer in 
	Go!)
		# Wow!  I would really like to use uname -r 
		if mount | grep -q /tmp/removable ; then
			mkdir -p /mnt/sysimage/tmp/removable
			device=`mount /tmp/removable | awk '{print $1}'`
			device_fs=`mount /tmp/removable | awk '{print $5}'`
			if [ -n "$device" -a -n "$device_fs" ] ; then
			    mount -t $device_fs $device /mnt/sysimage/tmp/removable
			fi
		fi
		mount devfs2 -t tmpfs /mnt/sysimage/dev
		rsync -aHl --devices --specials /dev /mnt/sysimage/
    		mount none -t proc /mnt/sysimage/proc
    		mount none -t sysfs /mnt/sysimage/sys
		cp -a /etc/lvm /mnt/sysimage/etc 2>/dev/null
		
		chattr -i /mnt/sysimage/sbin/mkinitrd
		chattr -i /mnt/sysimage/usr/share/man/man8/mkinitrd.8.gz
		cp -a patches/* /mnt/sysimage/
		chmod +x /mnt/sysimage/sbin/mkinitrd
		chattr +i /mnt/sysimage/sbin/mkinitrd
		chattr +i /mnt/sysimage/usr/share/man/man8/mkinitrd.8.gz
	
		for uname_r in `ls -1 /mnt/sysimage/lib/modules/` ; do
			PATH=$PATH:/sbin:/bin:/usr/sbin:/usr/bin chroot /mnt/sysimage /sbin/mkinitrd --force-scsi-probe --force-lvm-probe -f /boot/initrd-${uname_r}.img ${uname_r} && echo $uname_r OK
		done
		
		break 2
		;;
	check!)
		PS1="Final Check# " sh
		;;
	esac
done


echo "Be careful, if mkinitrd is updated, you may wind up with a rock!"
echo "You can exclude mkinitrd from your yum updates, or make it immutable"
echo "Either way you are on your own unless this patch makes it in!"
echo .
echo "Now, you can co back to Anaconda and select reboot."

# vim: ts=4:sw=4:norl
